@@ -1,3 +1,5 @@
+require v5.10;
+
 package Module::Extract::VERSION;
 use strict;
 
@@ -9,11 +11,13 @@ use vars qw($VERSION);
 
 use Carp qw(carp);
 
-$VERSION = '1.01';
+$VERSION = '1.111';
+
+=encoding utf8
 
 =head1 NAME
 
-Module::Extract::VERSION - Extract a module version without running code
+Module::Extract::VERSION - Extract a module version safely
 
 =head1 SYNOPSIS
 
@@ -29,7 +33,7 @@ Module::Extract::VERSION - Extract a module version without running code
 
 This module lets you pull out of module source code the version number
 for the module. It assumes that there is only one C<$VERSION>
-in the file.
+in the file and the entire C<$VERSION> statement is on the same line.
 
 =cut
 
@@ -41,7 +45,8 @@ in the file.
 
 Given a module file, return the module version. This works just like
 C<mldistwatch> in PAUSE. It looks for the single line that has the
-C<$VERSION> statement, extracts it, evals it, and returns the result.
+C<$VERSION> statement, extracts it, evals it in a Safe compartment,
+and returns the result.
 
 In scalar context, it returns just the version as a string. In list
 context, it returns the list of:
@@ -54,70 +59,87 @@ context, it returns the list of:
 
 =cut
 
-sub parse_version_safely # stolen from PAUSE's mldistwatch, but refactored
-	{
-	my $class = shift;
-	my $file = shift;
-	
+sub parse_version_safely { # stolen from PAUSE's mldistwatch, but refactored
+	my( $class, $file ) = @_;
+
 	local $/ = "\n";
 	local $_; # don't mess with the $_ in the map calling this
-	
+
 	my $fh;
-	unless( open $fh, "<", $file )
-		{
+	unless( open $fh, "<", $file ) {
 		carp( "Could not open file [$file]: $!\n" );
 		return;
 		}
-	
+
 	my $in_pod = 0;
-	my( $sigil, $var, $version, $line_number );
-	while( <$fh> ) 
-		{
+	my( $sigil, $var, $version, $line_number, $rhs );
+	while( <$fh> ) {
 		$line_number++;
-		#print STDERR "Read: $_";
 		chomp;
 		$in_pod = /^=(?!cut)/ ? 1 : /^=cut/ ? 0 : $in_pod;
 		next if $in_pod || /^\s*#/;
 
-		next unless /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/;
-		( $sigil, $var ) = ( $1, $2 );
-		
-		#print STDERR "Got $1 and $2\n";
-		
-		$version = $class->_eval_version( $_, $sigil, $var );
+		# package NAMESPACE VERSION  <-- we handle that
+		# package NAMESPACE VERSION BLOCK
+
+		next unless /
+			(?<sigil>
+				[\$*]
+			)
+			(?<var>
+				(?<package>
+					[\w\:\']*
+				)
+				\b
+				VERSION
+			)
+			\b
+			.*?
+			\=
+			(?<rhs>
+				.*
+			)
+			/x ||
+			m/
+			\b package \s+
+			(?<package> \w[\w\:\']* ) \s+
+			(?<rhs> \S+ ) \s* [;{]
+			/x;
+		( $sigil, $var, $rhs ) = @+{ qw(sigil var rhs) };
+
+		if ($sigil) {
+			$version = $class->_eval_version( $_, @+{ qw(sigil var rhs) } );
+		}
+		else {
+			$version = $class->_eval_version( $_, '$', 'VERSION', qq('$rhs') );
+		}
 
 		last;
 		}
 	$line_number = undef if eof($fh) && ! defined( $version );
 	close $fh;
-	
-	return wantarray ? ( $sigil, $var, $version, $file, $line_number ) : $version;
+
+	return wantarray ?
+		( $sigil, $var, $version, $file, $line_number )
+			:
+		$version;
 	}
 
-sub _eval_version
-	{
-	my $class = shift;
-	
-	my( $line, $sigil, $var ) = @_;
-	
-	#print STDERR "_eval_version called with @_\n";
-	
-	my $eval = qq{ 
-		package ExtUtils::MakeMaker::_version;
+sub _eval_version {
+	my( $class, $line, $sigil, $var, $rhs ) = @_;
 
-		local $sigil$var;
-		\$$var=undef; do {
-			$line
-			}; \$$var
-		};
-		
-	my $version = do {
-		local $^W = 0;
-		no strict;
-		eval( $eval );
-		};
+	require Safe;
+	require version;
+	local $^W = 0;
 
-	#print STDERR "Version is $version\n";
+	my $s = Safe->new;
+	$s->share_from('main', ['*version::']);
+	$s->share_from('version', ['&qv']);
+	if (defined $Devel::Cover::VERSION) {
+		$s->share_from('main', ['&Devel::Cover::use_file']);
+	}
+	$s->reval('$VERSION = ' . $rhs);
+	my $version = $s->reval('$VERSION');
 
 	return $version;
 	}
@@ -135,11 +157,14 @@ This code is in Github:
 brian d foy, C<< <bdfoy@cpan.org> >>
 
 I stole the some of this code from C<mldistwatch> in the PAUSE
-code by Andreas KE<ouml>nig, but I've moved most of it around.
+code by Andreas König, but I've moved most of it around.
+
+Andrey Starodubtsev added code to handle the v5.12 and v5.14
+C<package> syntax.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2008-2011, brian d foy, All Rights Reserved.
+Copyright © 2008-2016, brian d foy C<< <bdfoy@cpan.org> >>. All rights reserved.
 
 You may redistribute this under the same terms as Perl itself.
 
